package net.sf.r4h;

import java.io.Serializable;

/**
 * 
 * Supports all Hibernate ordering strategies and also provides {@link Order#nullsFirst()},
 * {@link Order#nullsLast()} support for databases that support such syntax.
 * 
 * Allows ordering using properties of associated entities without need of explicitly setting
 * association path.
 * 
 * Sorting is case sensitive by default.
 * 
 * Provides methods for sorting by popular fields such as id,name,date,time.
 * 
 * Can be used in environments(GWT,RMI,Web Services clients) without direct Hibernate dependencies.
 * 
 *  The class is immutable and thread safe.
 * 
 * @author vitaliy.se@gmail.com
 * @since 1.0.0
 * 
 */
public class Order implements Serializable {

	private static final String TIME = "time";

	private static final String DATE = "date";

	private static final String NAME = "name";

	private static final String ID = "id";

	private static final long serialVersionUID = -6294213434967991404L;

	private String sortedFieldPath;
	private SortingDirection sortingDirection;
	private CaseSensitivity caseSensitivity = CaseSensitivity.SENSITIVE;
	private NullsPosition nullsPosition = NullsPosition.NOT_SPECIFIED;

	public NullsPosition getNullsPosition() {
		return nullsPosition;
	}

	/**
	 * Needed for GWT RPC
	 */
	private Order() {
		this(null, null);
	}

	private Order(String sortedFieldPath, SortingDirection sortingDirection) {
		this.sortedFieldPath = sortedFieldPath;
		this.sortingDirection = sortingDirection;
	}

	public static Order asc(String sortedFieldPath) {
		return new Order(sortedFieldPath, SortingDirection.ASCENDING);
	}

	public static Order ascById() {
		return new Order(ID, SortingDirection.ASCENDING);
	}

	public static Order ascByName() {
		return new Order(NAME, SortingDirection.ASCENDING);
	}

	public static Order ascByDate() {
		return new Order(DATE, SortingDirection.ASCENDING);
	}

	public static Order ascByTime() {
		return new Order(TIME, SortingDirection.ASCENDING);
	}

	public static Order desc(String sortedFieldPath) {
		return new Order(sortedFieldPath, SortingDirection.DESCENDING);
	}

	public static Order descById() {
		return new Order(ID, SortingDirection.DESCENDING);
	}

	public static Order descByName() {
		return new Order(NAME, SortingDirection.DESCENDING);
	}

	public static Order descByDate() {
		return new Order(DATE, SortingDirection.DESCENDING);
	}

	public static Order descByTime() {
		return new Order(TIME, SortingDirection.DESCENDING);
	}

	public Order ignoreCase() {
		caseSensitivity = CaseSensitivity.INSENSITIVE;
		return this;
	}

	public CaseSensitivity getGetCaseSensitivity() {
		return caseSensitivity;
	}

	public String getSortedFieldPath() {
		return sortedFieldPath;
	}

	public SortingDirection getSortingDirection() {
		return sortingDirection;
	}

	public Order nullsFirst() {
		nullsPosition = NullsPosition.FIRST;
		return this;
	}

	public Order nullsLast() {
		nullsPosition = NullsPosition.LAST;
		return this;
	}

	/**
	 * Determines sort direction.
	 */
	public enum SortingDirection {
		ASCENDING, DESCENDING
	}

	/**
	 * Determines case sensitivity during sorting.
	 */
	public enum CaseSensitivity {
		SENSITIVE, INSENSITIVE
	}

	/**
	 * Determines nulls position during sorting.
	 */
	public enum NullsPosition {
		NOT_SPECIFIED, FIRST, LAST
	}
}
